from xml.etree import ElementTree as ET

from sigla.lib.Nodes.Node import Node
from sigla.lib.Nodes.NodeFile import NodeFile
from sigla.lib.Nodes.NodeRoot import NodeRoot
from sigla.lib.Nodes.NodeTemplate import NodeTemplate


def from_xml(node: ET.Element):
    attributes = Node.attributes_from_node(node)
    meta = Node.meta_from_node(node)

    children = []
    for child in node:
        children.append(from_xml(child))

    if node.tag == "file":
        return NodeFile(children=children, attributes=attributes, meta=meta)
    elif node.tag == "root":
        return NodeRoot(children=children, attributes=attributes, meta=meta)
    else:
        return NodeTemplate(
            children=children, attributes=attributes, meta=meta
        )



import os

from sigla.lib.Nodes.Node import Node
from sigla.lib.helpers.files import ensure_parent_dir
from sigla.lib.helpers.Context import Context


class NodeFile(Node):
    kind = "file"

    def process(self, ctx=None):
        if ctx is None:
            ctx = Context()

        if not self.name:
            raise Exception("# No name attached to the file element.")

        ctx.push_context(self)

        text = ""
        for child in self:
            text += child.process(ctx)

        ctx.pop_context()

        path = os.path.join(os.getcwd(), self.name)

        ensure_parent_dir(path)

        with open(path, "w") as h:
            h.write(text)



import configparser
from os.path import join

import os
from typing import Optional

BASE_PATH = os.getcwd()


class Config(configparser.ConfigParser):
    base_path: Optional[str] = None
    sigma_path: Optional[str] = None
    config_filename: Optional[str] = None

    def __init__(
        self,
        sigma_path=".sigla",
        config_filename="config.ini",
        *args,
        **kwargs
    ):
        super().__init__(*args, **kwargs)

        self.base_path = BASE_PATH
        self.sigma_path = join(self.base_path, sigma_path)
        self.config_filename = config_filename

        self.read(self.config_path)

        #
        # sigla
        #
        if not self.has_section("sigla"):
            self["sigla"] = {}

        self["sigla"]["templates"] = self["sigla"].get(
            "templates", ".sigla/templates"
        )
        self["sigla"]["snapshots"] = self["sigla"].get(
            "snapshots", ".sigla/snapshots"
        )
        self["sigla"]["definitions"] = self["sigla"].get(
            "definitions", ".sigla/definitions"
        )

        #
        # templates
        #
        if not self.has_section("templates"):
            self["templates"] = {}

        self["templates"]["create_missing_templates"] = str(
            self.create_missing_templates
        )

    @property
    def create_missing_templates(self):
        return self["templates"].getboolean(
            "create_missing_templates", fallback=True
        )

    @property
    def config_path(self):
        return os.path.join(self.sigma_path, self.config_filename)

    @property
    def templates_path(self):
        return join(self.base_path, self["sigla"]["templates"])

    @property
    def definitions_path(self):
        return join(self.base_path, self["sigla"]["definitions"])

    @property
    def snapshots_path(self):
        return join(self.base_path, self["sigla"]["snapshots"])

    def save(self):
        with open(self.config_path, "w") as configfile:
            self.write(configfile)



from sigla.lib.Nodes.Node import Node
from sigla.lib.helpers.Context import Context


class NodeRoot(Node):
    kind = "root"

    def process(self, ctx=None):
        return list(map(lambda e: e.process(ctx), self.children))



from pprint import pprint
from typing import Optional

import frontmatter
import json
import os
from pathlib import Path
import pydash as _
from frontmatter import u

from sigla.lib import Node
from sigla.lib.Nodes.Node import Node
from sigla.lib.Nodes.template.engines.njk import njk
from sigla.lib.helpers.Context import Context
from sigla.lib.helpers.files import ensure_parent_dir


#
# frontmatter utils
#
def fm_split(text, encoding="utf-8", handler=None):
    text = u(text, encoding).strip()

    # this will only run if a handler hasn't been set higher up
    handler = handler or frontmatter.detect_format(text, frontmatter.handlers)
    if handler is None:
        return None, text, None

    try:
        fm, content = handler.split(text)
    except ValueError:
        return None, text, handler

    return fm, content, handler


def fm_parse_fm(fm, handler, metadata=None):
    if metadata is None:
        metadata = {}

    fm = handler.load(fm)
    if isinstance(fm, dict):
        metadata.update(fm)

    return metadata


#
#
#


def get_default_template_content(context):
    def default_njk_template(dumped_context):
        return f"""
    
    Available vars: {dumped_context}
    
    Handle children:
    
    {{{{ render(children) }}}}
    
    {{% for child in children %}}
        {{{{ render(child) }}}}
    {{% endfor %}}
    
    """

    json_context = json.dumps(list(context.keys()) + ["children"])
    return default_njk_template(json_context)


class NodeTemplate(Node):
    template: Optional[str] = None
    kind = "template"
    templates_directory = "./.sigla/templates"
    create_missing_templates = True

    @property
    def template_name(self):
        name = self.get_template_name()
        name = name.replace("-", "/")
        name = f"{name}.njk"
        return name

    @property
    def template_path(self):
        return Path(os.path.join(self.templates_directory, self.template_name))

    #
    # metadata related stuff
    #
    def get_children_metadata(self, ctx):
        children = self.children

        if len(children) == 0:
            return None

        child: NodeTemplate
        metadata_ = []
        for child in children:
            if child.get_metadata:
                ctx.push_context(child)
                metadata_.append(child.get_metadata(ctx))
                ctx.pop_context()

        #
        metadata = _.filter_(metadata_, lambda x: x is not None)
        if len(metadata) > 0:
            return metadata

    def get_self_metadata(self, ctx):
        template = self.get_raw_template()
        if not template:
            return
        fm_raw, __, handler = fm_split(template)
        metadata = fm_parse_fm(self.render(fm_raw, ctx), handler) if fm_raw and handler else None
        return metadata

    def get_metadata(self, ctx):
        metadata = self.get_self_metadata(ctx)
        children_metadata = self.get_children_metadata(ctx)

        if metadata is None and children_metadata is None:
            return None
        elif not children_metadata:
            return [metadata]
        else:
            return [metadata, children_metadata]

    def ensure_template(self, default_value):
        """ Ensure the file really exists """

        if self.get_raw_template() is not None:
            return

        if not self.create_missing_templates:
            return

        ensure_parent_dir(self.template_path)
        if self.template_path.exists():
            return
        with open(self.template_path, "w") as h:
            h.write(default_value)

    def get_raw_template(self):
        try:
            with open(self.template_path, 'r') as h:
                return h.read()
        except FileNotFoundError:
            pass

    def render(self, template, ctx, **kwargs):

        if ctx is None:
            ctx = Context()

        def f(e):
            return e.process(ctx)

        def wrapped(node: Node, sep="\n"):
            if type(node) == list:
                nodes = map(f, node)
                return sep.join(nodes)
            return f(node)

        context = ctx.get_context()

        result = njk(
            template,
            **context,
            ctx=context,
            children=self.children,
            render=wrapped,
            **kwargs,
        )

        return result

    def process(self, ctx=None):

        if ctx is None:
            ctx = Context()
        ctx.push_context(self)

        self.ensure_template(get_default_template_content(ctx.get_context()))

        fm_raw, body_raw, handler = fm_split(self.get_raw_template())

        metadata = self.get_children_metadata(ctx)

        #
        res = self.render(body_raw, ctx, meta=metadata)
        ctx.pop_context()

        return res



from os.path import join
from pathlib import Path

import click

# import logging

from sigla import __version__
from sigla.lib.helpers.files import ensure_dirs, ensure_parent_dir
from sigla.main import run
from sigla.lib.Config import Config
from sigla.cli.SnapshotCli import SnapshotCli

# logging.basicConfig(
#     level=logging.INFO,
#     format="[%(levelname)s] %(message)s",
#     handlers=[
#         logging.FileHandler("debug.log"),
#         # logging.StreamHandler()
#     ],
# )

config = Config()


# @click.option("--debug/--no-debug", default=False)
# click.echo('Debug mode is %s' % ('on' if debug else 'off'))
@click.group()
def cli():
    pass


@cli.command()
def init():
    ensure_dirs(
        config.sigma_path,
        config.templates_path,
        config.snapshots_path,
        config.definitions_path,
    )
    config.save()


@cli.command()
@click.argument("name", type=click.types.STRING)
def new_definition(name):
    p = Path(join(config.definitions_path, name + ".xml"))
    ensure_parent_dir(p)
    name = name.replace("/", "-")

    if p.exists():
        raise Exception("This definition already exists")

    # if file exists throw error
    with open(p, "w") as h:
        h.write(
            f"""<root>
    <file name="output/{name}[.ext]">
        <{name}>
        </{name}>
    </file>
</root>"""
        )


@cli.command()
@click.argument("name", type=click.types.STRING)
def run_definition(name):
    p = Path(join(config.definitions_path, name + ".xml"))
    if not p.exists():
        raise Exception("This definition does not exists")
    run(file=p)


@cli.command()  # @cli, not @click!
@click.argument("files", nargs=-1, type=click.Path(exists=True))
def render(files):
    for file in files:
        run(file=file)


@cli.command()
def version():
    """
    Prints the current version
    """
    click.echo(f"Version: {__version__}")


@cli.command()
@click.argument("files", nargs=-1, type=click.Path(exists=True))
def snapshot(files):
    """
    Makes snapshots
    """
    for file in files:
        snap = SnapshotCli(file)
        snap.make_snapshots()


@cli.command()
@click.argument("files", nargs=-1, type=click.Path(exists=True))
def verify(files):
    """
    Verify snapshots
    """
    for file in files:
        snap = SnapshotCli(file)
        snap.verify_snapshots()


if __name__ == "__main__":
    cli()



from pathlib import Path


def ensure_dirs(*paths):
    for path_ in paths:
        Path(path_).mkdir(parents=True, exist_ok=True)


def ensure_parent_dir(*paths):
    for path_ in paths:
        Path(path_).parent.mkdir(parents=True, exist_ok=True)
